import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Map<K,V> implements Iterable<Tuple<K,V>> {
  private List<Tuple<K,V>> map;

  public Map() {
    map = new ArrayList<>();
  }

  public Either<String,V> lookup(K key) {
    for (Tuple<K,V> t : map)
      if (t.fst().equals(key))
        return Either.right(t.snd());

    return Either.left("Key not found!");
  }

  public boolean member(K key) {
    return lookup(key).isRight();
  }

  public void insert(K key, V value) {
    if (!member(key))
      map.add(new Tuple(key, value));
  }

  public void remove(K key) {
    map = Lists.filter(k -> !k.fst().equals(key), map);
  }

  @Override
  public Iterator<Tuple<K, V>> iterator() {
    return new Iterator<Tuple<K, V>>() {
      private int index = 0;

      @Override
      public boolean hasNext() {
        return index < map.size();
      }

      @Override
      public Tuple<K, V> next() {
        return map.get(index++);
      }
    };
  }
}
